"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const http = __importStar(require("http"));
const https = __importStar(require("https"));
const url = __importStar(require("url"));
const zlib = __importStar(require("zlib"));
const readline = __importStar(require("readline"));
const stream = __importStar(require("stream"));
const cReconnectionDelay = 60000;
class Client extends stream.Readable {
    constructor(connection) {
        super({
            objectMode: true,
        });
        this.connection = connection;
        this._running = false;
        this._request = null;
        this._response = null;
    }
    start() {
        if (!this._running) {
            this._running = true;
            this.runStream();
            return true;
        }
        return false;
    }
    stop() {
        if (this._running) {
            this._running = false;
            clearTimeout(this._tidReconnect);
            if (this._request) {
                this._request.abort();
            }
            return true;
        }
        return false;
    }
    _read() {
        if (this._response && this._response.isPaused()) {
            this._response.resume();
        }
    }
    ;
    runStream() {
        const link = `https://${this.connection.customerName}.${this.connection.domain ? this.connection.domain : 'socialgist.com'}/stream/${this.connection.dataSource}_${this.connection.streamName}/subscription/${this.connection.subscriptionName}/part/1/data.json`;
        const parsedURL = url.parse(link);
        const r = (parsedURL.protocol === 'https:') ? https.request : http.request;
        const req = r({
            auth: `${this.connection.username}:${this.connection.password}`,
            protocol: parsedURL.protocol,
            host: parsedURL.hostname,
            port: parsedURL.port ? +parsedURL.port : undefined,
            method: 'GET',
            path: parsedURL.path,
        }, (res) => {
            if (res.statusCode !== 200) {
                this.emit('error', `Invalid response status: ${res.statusCode}`);
            }
            else {
                this._response = res;
                let decodedRes;
                const zlibOptions = {
                    flush: zlib.Z_SYNC_FLUSH,
                    finishFlush: zlib.Z_SYNC_FLUSH,
                };
                switch (res.headers['content-encoding']) {
                    case 'gzip':
                        decodedRes = res.pipe(zlib.createGunzip(zlibOptions));
                        break;
                    case 'deflate':
                        decodedRes = res.pipe(zlib.createInflate(zlibOptions));
                        break;
                    default:
                        decodedRes = res;
                        break;
                }
                const rl = readline.createInterface({
                    input: decodedRes,
                });
                rl.on('line', (line) => {
                    const p = this.push(line);
                    if (p) {
                        if (res.isPaused()) {
                            res.resume();
                        }
                    }
                    else {
                        if (!res.isPaused()) {
                            res.pause();
                        }
                    }
                });
                res.on('close', () => {
                    this._response = null;
                });
            }
        });
        req.on('close', () => {
            this._request = null;
            if (this._running) {
                this.emit('error', 'Connection closed');
                this._tidReconnect = setTimeout(() => this.runStream(), cReconnectionDelay);
            }
        });
        req.end();
        this._request = req;
    }
}
exports.Client = Client;
//# sourceMappingURL=index.js.map